import { HttpException, Injectable, Logger } from '@nestjs/common';
import * as querystring from 'node:querystring';
import { HttpService } from '@nestjs/axios';
import { firstValueFrom } from 'rxjs';
import { AuthenticateUser } from '../../entity/authentication-user.entity';
import { decode } from 'jsonwebtoken';
import { errorMessage } from 'apps/middleware/src/exception.filter';

@Injectable()
export class KeycloakAuthenticationService {
  private readonly httpService: HttpService;

  private readonly logger = new Logger(KeycloakAuthenticationService.name);

  constructor(private readonly _httpService: HttpService) {
    this.httpService = _httpService;
  }

  async authenticateUserviaKeycloak(
    username: string,
    password: string
  ): Promise<AuthenticateUser> {
    try {
      const body = {
        grant_type: 'password',
        client_id: process.env.KEYCLOAK_CLIENT,
        client_secret: process.env.KEYCLOAK_CLIENT_SECRET,
        username: username,
        password: password,
      };

      const urlEncodedBody = querystring.encode(body);

      //Form content type
      const contentType = {
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
        },
      };
      const userSessionsURL = process.env.KEYCLOAK_USERSESSION_URL;

      // Calling keycloak user session endpoint
      const userSessionResultAsObservable = this.httpService.post(
        userSessionsURL,
        urlEncodedBody,
        contentType
      );

      // Response from keycloak for user session
      const result = await firstValueFrom(userSessionResultAsObservable);

      let userSession = {
        access_token: result.data.access_token,
        expires_in: result.data.expires_in,
        refresh_expires_in: result.data.refresh_expires_in,
        refresh_token: result.data.refresh_token,
        token_type: result.data.token_type,
        scope: result.data.scope,
      };

      const authenticationTokenPayload: any = decode(userSession.access_token);

      const user = {
        userUUID: authenticationTokenPayload.userUUID,
        email: authenticationTokenPayload.email,
        username: authenticationTokenPayload.preferred_username,
        firstName: authenticationTokenPayload.given_name,
        lastName: authenticationTokenPayload.family_name,
        fullName: authenticationTokenPayload.name,
      };

      let authenticateUserResponse = {
        ...userSession,
        ...user,
      };

      return authenticateUserResponse;
    } catch (error) {
      const errorResponse = errorMessage(error);

      throw new HttpException(errorResponse, Number(errorResponse.statusCode));
    }
  }
}
