import {
  ExecutionContext,
  HttpException,
  HttpStatus,
  Injectable,
  SetMetadata,
} from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { AuthGuard, PassportStrategy } from '@nestjs/passport';
import { passportJwtSecret } from 'jwks-rsa';
import { ExtractJwt, Strategy } from 'passport-jwt';

@Injectable()
export class JwtTokenExtract extends PassportStrategy(Strategy) {
  constructor() {
    /**
     * Token validation
     */
    super({
      //To configure identity provider details
      secretOrKeyProvider: passportJwtSecret({
        jwksRequestsPerMinute: 5,
        jwksUri: process.env.KEYCLOAK_JWKS_URI,
      }),
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      issuer: process.env.KEYCLOAK_ISSUER_URL,
      algorithms: ['RS256'],
    });
  }

  validate(payload: JSON) {
    return payload;
  }
}

/**
 * @summary
 * To allow unauthentic user setup
 * reflector - Retrieve metadata for a specified key for a specified target
 * getHandler - Handler (method) that will be invoked next in the request pipeline
 */
export class JwtAuthGuard extends AuthGuard('jwt') {
  constructor(private readonly reflector: Reflector) {
    super();
  }

  handleRequest(err, user, info, context: ExecutionContext) {
    const request = context.switchToHttp().getRequest();

    const allowAny = this.reflector.get<string[]>(
      'allow-any',
      context.getHandler()
    );

    if (allowAny) return true;

    if (user) return user;

    throw new HttpException(
      {
        statusCode: HttpStatus.UNAUTHORIZED,
        message: 'Unauthorized',
        error: 'Unauthorized',
      },
      HttpStatus.UNAUTHORIZED
    );
  }
}

export const UnProtected = () => SetMetadata('allow-any', true);
