import { ApiProperty } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import { IsNumber, Max, Min } from 'class-validator';

/**
 * Basic View Report DTO that will be inherited by individual entities
 *
 */
export class ViewEntityRequestDto {
  /**
   * Search Query
   *
   * Will search the entity for given search term
   *
   * @example
   * bluebird
   */
  @ApiProperty({
    required: false,
    description: 'Search query',
    default: '',
  })
  q: string = '';

  /**
   * Number of records to rtrieve from the database
   *
   * @example
   * 20
   */
  @ApiProperty({
    required: true,
    description: 'Number of records to retrieve from database',
    default: 20,
  })
  @Type(() => Number)
  @IsNumber()
  @Min(1)
  @Max(500)
  limit: number;

  /**
   * Number of records to skip as part of pagination
   *
   * If you want to skip 2 pages and view 3rd page, and if the limit is 20, then set offset to 40.
   * Default will always be 0.
   *
   * @example
   * 40
   */
  @ApiProperty({
    required: true,
    description: 'Number of records to skip as part of pagination',
    default: 0,
  })
  @Type(() => Number)
  @IsNumber()
  @Min(0)
  @Max(10000)
  offset: number;

  /**
   * Sort column and sort order
   *
   * Specify the column to be sorted along with ascending or descending
   *
   * @example
   * modifiedTime desc
   */
  @ApiProperty({
    required: false,
    description: 'Sort column and sort order',
    default: '',
  })
  sort: string;
}
