/* eslint-disable no-case-declarations */
import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import * as expressions from 'angular-expressions';
import { Store } from '@ngrx/store';
import { AppStoreSelectors } from '@hash8io/core-web';

@Injectable({
  providedIn: 'root',
})
export class DataGridWebService {
  // Search
  private _searchText: BehaviorSubject<string> = new BehaviorSubject('');
  searchText: Observable<string> = this._searchText.asObservable();

  constructor(private store: Store) {
    //
  }

  evaluate(expressionOrFunc: any, params: any) {
    if (typeof expressionOrFunc === 'function') {
      // valueGetter is a function, so just call it
      const func = expressionOrFunc;
      return func(params);
    } else if (typeof expressionOrFunc === 'string') {
      // valueGetter is an expression, so execute the expression
      const expression = expressionOrFunc;
      return this.evaluateExpression(expression, params);
    } else {
      console.error(
        'H8 Grid: value should be either a string or a function',
        expressionOrFunc
      );
    }
  }

  evaluateExpression(expression: any, params: any) {
    try {
      const javaScriptFunction = this.createExpressionFunction(expression);
      // the params don't have all these values, rather we add every possible
      // value a params can have, which makes whatever is in the params available.
      const result = javaScriptFunction(
        params.value,
        params.context,
        params.oldValue,
        params.newValue,
        params.value,
        params.node,
        params.data,
        params.colDef,
        params.rowIndex,
        params.api,
        params.columnApi,
        params.getValue,
        params.column,
        params.columnGroup
      );
      return result;
    } catch (e) {
      // the expression failed, which can happen, as it's the client that
      // provides the expression. so print a nice message
      // tslint:disable-next-line
      // console.log('Processing of the expression failed');
      // tslint:disable-next-line
      // tslint:disable-next-line
      // tslint:disable-next-line
      return null;
    }
  }

  createExpressionFunction(expression: any) {
    // if not found in cache, return the function
    const functionBody = this.createFunctionBody(expression);
    const theFunction = new Function(
      'x, ctx, oldValue, newValue, value, node, data, colDef, rowIndex, api, columnApi, getValue, column, columnGroup',
      functionBody
    );

    return theFunction;
  }

  createFunctionBody(expression: any) {
    // if the expression has the 'return' word in it, then use as is,
    // if not, then wrap it with return and ';' to make a function
    if (expression.indexOf('return') >= 0) {
      return expression;
    } else {
      return 'return ' + expression + ';';
    }
  }

  searchGrid(q: string) {
    this._searchText.next(q);
  }

  computeLabelExpression(config: any, value: any, rowData: any) {
    let label = '';
    let color = '';
    let tooltip = '';
    let selectedOption = {};
    if (config?.options) {
      const fieldOptions = config?.options;
      fieldOptions?.forEach((x: any) => {
        if (
          (value + '')?.toLowerCase() ===
          ((x?.value + '')?.toLowerCase() || (x?.label + '')?.toLowerCase())
        ) {
          label = x.label;
          color = x.color;
          selectedOption = x;
          tooltip = x.tooltip;
        }
      });
      if (!label) {
        label = value;
        if (label) {
          color = config?.defaultColor;
        }
      }
      // console.log(label, color);
    }
    return {
      label: label,
      color: color,
      tooltip: tooltip,
      ...selectedOption,
    };
  }

  computeToggleExpression(config: any, value: any, rowData: any) {
    let label = '';
    let color = '';
    let tooltip = '';
    let selectedOption = {};
    let valueExist = false;
    if (config) {
      const fieldOptions = config;
      fieldOptions?.forEach((x: any) => {
        if (
          (value + '')?.toLowerCase() ===
          ((x?.value + '')?.toLowerCase() || (x?.label + '')?.toLowerCase())
        ) {
          if (x?.on) {
            const evaluate = expressions.compile(x.on);
            if (evaluate(rowData)) {
              label = x.label;
              color = x.color;
              selectedOption = x;
              tooltip = x.tooltip;
              valueExist = true;
            }
          } else {
            label = x.label;
            color = x.color;
            selectedOption = x;
            tooltip = x.tooltip;
            valueExist = true;
          }
        }
      });
    }
    if (valueExist) {
      return {
        label: label,
        color: color,
        tooltip: tooltip,
        ...selectedOption,
      };
    } else {
      return null;
    }
  }

  //   getGridHeight(
  //     gridApi: any,
  //     rowsCount: number,
  //     heightType = 'fixed',
  //     customHeight = '500px',
  //     autoRowsLimit = 13
  //   ) {
  //     if (heightType === 'fixed') {
  //       gridApi.setDomLayout('normal');
  //       return customHeight;
  //     } else if (heightType === 'auto') {
  //       if (rowsCount > gridApi.paginationGetPageSize()) {
  //         gridApi.setDomLayout('autoHeight');
  //         return '';
  //       } else if (gridApi.paginationGetPageSize() >= autoRowsLimit) {
  //         gridApi.setDomLayout('normal');
  //         // const height = Number(autoRowsLimit) * 40 + 100 + 'px';
  //         // return height;
  //         return 'calc(80vh - 64px)';
  //       } else {
  //         gridApi.setDomLayout('autoHeight');
  //         return '';
  //       }
  //     } else if (heightType === 'fill') {
  //       gridApi.setDomLayout('normal');
  //       return 'calc(80vh - 64px)';
  //     } else if (typeof heightType === 'string') {
  //       return heightType;
  //     } else {
  //       return customHeight; // Default to '250px'
  //     }
  //   }
  // }

  getGridHeight(
    gridApi: any,
    rowsCount: number,
    heightType = 'fill',
    negativeSpace = '64px',
    customHeight = '500px',
    autoRowsLimit = 13
  ) {
    const headerSpace = '52px';
    const footerSpace = '35px';
    const toolbarSpace = '46px';
    const _negativeSpace = `calc(${negativeSpace} + ${headerSpace} + ${footerSpace} + ${toolbarSpace})`;
    const fillHeight = `calc(100vh - ${_negativeSpace})`;

    switch (heightType) {
      case 'fixed':
        gridApi.setDomLayout('normal');
        return customHeight;

      case 'auto':
        const pageSize = gridApi.paginationGetPageSize();
        let gridHeight = '';
        if (rowsCount < pageSize && rowsCount <= autoRowsLimit) {
          gridApi.setDomLayout('autoHeight');
          gridHeight = '';
        } else if (pageSize >= autoRowsLimit) {
          // const height = Number(autoRowsLimit) * 40 + 100 + 'px';
          // return height;
          gridApi.setDomLayout('normal');
          gridHeight = fillHeight;
        } else {
          gridApi.setDomLayout('autoHeight');
          gridHeight = '';
        }
        return gridHeight;

      case 'fill':
        gridApi.setDomLayout('normal');
        return fillHeight;

      default:
        if (typeof heightType === 'string' && heightType.endsWith('px')) {
          return typeof heightType === 'string' ? heightType : customHeight;
        } else if (typeof heightType === 'number') {
          return heightType + 'px';
        } else {
          return customHeight;
        }
    }
  }
  public dataUnitConversion(value: any, options?: any) {
    let formattedValue: any;
    this.store
      .select(AppStoreSelectors.selectDisplaySettings)
      .subscribe((preferences: any) => {
        if (value && value != '-') {
          switch (options?.measurementUnit?.toLowerCase()) {
            case 'temperature':
              switch (preferences.temperature?.toLowerCase()) {
                case 'celsius':
                  formattedValue = value;
                  break;
                case 'fahrenheit':
                  formattedValue = (value * (9 / 5) + 32).toFixed(
                    options?.floatPrecision || 2
                  );
                  break;
                case 'kelvin':
                  formattedValue = (value + 273.15).toFixed(
                    options?.floatPrecision || 2
                  );
                  break;
                default:
                  formattedValue = value;
                  break;
              }
              break;
            case 'distance':
              break;
            case 'scale':
              break;
            case 'quantity':
              break;
            default:
              formattedValue = value;
          }
        } else {
          formattedValue = '-';
        }
      });

    return `${formattedValue}`;
  }
}
