import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsWebComponent } from './forms-web.component';
import { SharedModule } from '@hash8io/core-web';
import { FormlyFieldConfig, FormlyModule } from '@ngx-formly/core';
import { FormlyPresetModule } from '@ngx-formly/core/preset';
import { FormlyMaterialModule } from '@ngx-formly/material';
import { SingleStepFormComponent } from './components/single-step-form/single-step-form.component';
import { MultiStepFormComponent } from './components/multi-step-form/multi-step-form.component';
import { TitleModule } from '@hash8io/core-web/title';
import { BannerModule } from '@hash8io/core-web/banner';
import { NgxMaskModule } from 'ngx-mask';
import {
  EmailValidator,
  fieldMatchValidator,
  PasswordValidator,
} from './forms.validators';
import {
  MatFormFieldDefaultOptions,
  MAT_FORM_FIELD_DEFAULT_OPTIONS,
} from '@angular/material/form-field';
import { FormlyMatSliderModule } from '@ngx-formly/material/slider';
import { MatNativeDateModule } from '@angular/material/core';
import { FormlyMatDatepickerModule } from '@ngx-formly/material/datepicker';
import { FormlyMatToggleModule } from '@ngx-formly/material/toggle';
import {
  FieldWrapperComponent,
  FormFieldType,
  FormFieldWrapperComponent,
} from './components';
import { FORM_FIELD_PRESETS } from './presets';
import {
  DropdownPanelContainer,
  FORM_FIELD_TYPES,
} from './components/field-types';
import { StepperType } from './components/field-types/stepper/stepper.type';
import { LabelType } from './components/field-types/label/label.type';
import { LoadingBarLibraryModule } from '@hash8io/core-web/loading-bar';
import { SingleStepFormService } from './components/single-step-form/service/single-step-form.service';
import { StepperFormComponent } from './components/stepper-form/stepper-form.component';
import { NgxMatIntlTelInputComponent } from 'ngx-mat-intl-tel-input';
import { HierarchyModule } from '@hash8io/core-web/hierarchy';
import { DropdownTriggerForDirective } from './components/field-types/dropdown/dropdown-trigger-for.directive';
import { DataReportWebModule } from '@hash8io/data-report-web';

@NgModule({
  declarations: [
    FormsWebComponent,
    SingleStepFormComponent,
    MultiStepFormComponent,
    FormFieldWrapperComponent,
    FieldWrapperComponent,
    ...FORM_FIELD_TYPES,
    StepperType,
    LabelType,
    StepperFormComponent,
    DropdownTriggerForDirective,
    DropdownPanelContainer,
  ],
  imports: [
    CommonModule,
    SharedModule,
    TitleModule,
    BannerModule,
    DataReportWebModule,
    NgxMaskModule.forRoot(),
    FormlyModule.forRoot({
      wrappers: [
        { name: 'panel', component: FormFieldWrapperComponent },
        { name: 'field', component: FieldWrapperComponent },
      ],
      types: [
        { name: 'password', component: FormFieldType.PasswordType },
        { name: 'phoneNumber', component: FormFieldType.PhoneNumberType },
        { name: 'odometer', component: FormFieldType.OdometerType },
        {
          name: 'textbox-async',
          component: FormFieldType.TextboxValidatorAsyncType,
        },
        { name: 'textbox', component: FormFieldType.TextboxType },
        { name: 'stepper', component: FormFieldType.StepperType },
        { name: 'textArea', component: FormFieldType.TextareaType },
        { name: 'checkBox', component: FormFieldType.CheckboxType },
        { name: 'multiCheckbox', component: FormFieldType.MultiCheckboxType },
        { name: 'selectType', component: FormFieldType.SelectType },
        { name: 'dropdown', component: FormFieldType.DropdownType },
        { name: 'toggleType', component: FormFieldType.ToggleType },
        { name: 'sliderType', component: FormFieldType.SliderType },
        { name: 'radioType', component: FormFieldType.RadioType },
        { name: 'repeatType', component: FormFieldType.RepeatType },
        { name: 'repeat', component: FormFieldType.RepeatType },
        { name: 'repeatGrid', component: FormFieldType.RepeatGridType },
        { name: 'label', component: FormFieldType.LabelType },
        { name: 'addEntity', component: FormFieldType.ReportType },
      ],
      presets: [...FORM_FIELD_PRESETS],
      validators: [
        { name: 'fieldMatch', validation: fieldMatchValidator },
        { name: 'email', validation: EmailValidator },
        { name: 'password', validation: PasswordValidator },
      ],
      validationMessages: [
        { name: 'email', message: 'Invalid email' },
        { name: 'required', message: 'This field is required' },
        {
          name: 'password',
          message: `Password should meet following conditions`,
        },
        {
          name: 'minLength',
          message: (error: any, field: FormlyFieldConfig) =>
            `Should have atleast ${field?.props?.minLength} characters`,
        },
        {
          name: 'maxLength',
          message: (error: any, field: FormlyFieldConfig) =>
            `Should have atleast ${field?.props?.maxLength} characters`,
        },
        {
          name: 'min',
          message: (error: any, field: FormlyFieldConfig) =>
            `Should be minimum ${field?.props?.min}`,
        },
        {
          name: 'max',
          message: (error: any, field: FormlyFieldConfig) =>
            `Should be maximum ${field?.props?.max}`,
        },
      ],
      extras: { lazyRender: true, resetFieldOnHide: true },
    }),
    FormlyPresetModule,
    FormlyMaterialModule,
    FormlyMatSliderModule,
    MatNativeDateModule,
    FormlyMatDatepickerModule,
    FormlyMatToggleModule,
    LoadingBarLibraryModule,
    NgxMatIntlTelInputComponent,
    HierarchyModule,
  ],
  exports: [
    FormsWebComponent,
    SingleStepFormComponent,
    MultiStepFormComponent,
    FormFieldWrapperComponent,
    FieldWrapperComponent,
    ...FORM_FIELD_TYPES,
    StepperType,
    LabelType,
    StepperFormComponent,
    DropdownPanelContainer,
    DropdownTriggerForDirective,
  ],
  providers: [
    SingleStepFormService,
    {
      provide: MAT_FORM_FIELD_DEFAULT_OPTIONS,
      useValue: {
        appearance: 'outline',
      } as MatFormFieldDefaultOptions,
    },
  ],
})
export class FormsWebModule {}
