import {
  Component,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
  TemplateRef,
} from '@angular/core';
import { isNil, includes } from 'lodash';
import { TreeviewItem, TreeviewSelection } from '../../models/tree-view-item';
import { TreeviewConfig } from '../../models/tree-view-config';
import { TreeviewItemTemplateContext } from '../../models/tree-view-item-template-context';
import { TreeviewHeaderTemplateContext } from '../../models/tree-view-header-template-context';
import { TreeviewI18n } from '../../models/tree-view-i18n';
import { TreeviewEventParser } from '../../helpers/treeview-event-parser';
import { TreeviewHelper } from '../../helpers/treeview-helper';
import { Subject } from 'rxjs';

class FilterTreeviewItem extends TreeviewItem {
  private readonly refItem: TreeviewItem;
  constructor(item: TreeviewItem) {
    super({
      text: item.text,
      value: item.value,
      iconUrl: item.iconUrl,
      disabled: item.disabled,
      checked: item.checked,
      collapsed: item.collapsed,
      children: item.children,
    });
    this.refItem = item;
  }

  updateRefChecked(): void {
    this.children.forEach((child) => {
      if (child instanceof FilterTreeviewItem) {
        child.updateRefChecked();
      }
    });

    let refChecked = this.checked;
    if (refChecked) {
      for (const refChild of this.refItem.children) {
        if (!refChild.checked) {
          refChecked = false;
          break;
        }
      }
    }
    this.refItem.checked = refChecked;
  }
}

@Component({
  selector: 'hash8-platform-tree-view',
  templateUrl: './tree-view.component.html',
  styleUrls: ['./tree-view.component.scss'],
})
export class TreeViewComponent implements OnInit, OnChanges {
  @Input() headerTemplate!: TemplateRef<TreeviewHeaderTemplateContext>;
  @Input() itemTemplate!: TemplateRef<TreeviewItemTemplateContext>;
  @Input() items!: TreeviewItem[];
  @Input() config: TreeviewConfig;
  @Input() label!: string;
  @Output() selectedChange = new Subject<any>();
  @Output() filterChange = new Subject<string>();
  headerTemplateContext!: TreeviewHeaderTemplateContext;
  allItem: TreeviewItem;
  filterText = '';
  filterItems!: TreeviewItem[];
  selection!: TreeviewSelection;

  constructor(
    public i18n: TreeviewI18n,
    private defaultConfig: TreeviewConfig,
    private eventParser: TreeviewEventParser
  ) {
    this.config = this.defaultConfig;
    this.allItem = new TreeviewItem({ text: 'All', value: undefined });
  }

  get hasFilterItems(): boolean {
    return !isNil(this.filterItems) && this.filterItems.length > 0;
  }

  get maxHeight(): string {
    return `${this.config.maxHeight}`;
  }

  ngOnInit(): void {
    this.createHeaderTemplateContext();
    this.generateSelection();
  }

  ngOnChanges(changes: SimpleChanges): void {
    const itemsSimpleChange = changes['items'];

    // DISABLED - Due to repeated second call after first call
    // if (this.items) {
    //   this.items.forEach((item) => {
    //     this.setCheckedRecursive(item);
    //   });
    // }

    if (!isNil(itemsSimpleChange) && !isNil(this.items)) {
      this.updateFilterItems();
      this.updateCollapsedOfAll();
      this.raiseSelectedChange();
    }

    const selectedItem = TreeviewHelper.findItemInList(this.filterItems, 9);
  }

  setCheckedRecursive(item: any) {
    if (item) {
      item.checked = true;
      if (item.children && Array.isArray(item.children)) {
        item.children.forEach((child: any) => {
          this.setCheckedRecursive(child);
        });
      }
    }
  }

  onAllCollapseExpand(): void {
    this.allItem.collapsed = !this.allItem.collapsed;
    this.filterItems.forEach((item) =>
      item.setCollapsedRecursive(this.allItem.collapsed)
    );
  }

  onFilterTextChange(text: string): void {
    this.filterText = text;
    this.filterChange.next(text);
    this.updateFilterItems();
  }

  onAllCheckedChange(): void {
    const checked = this.allItem.checked;
    this.filterItems.forEach((item) => {
      item.setCheckedRecursive(checked);
      if (item instanceof FilterTreeviewItem) {
        item.updateRefChecked();
      }
    });

    this.raiseSelectedChange();
  }

  onItemCheckedChange(item: TreeviewItem, checked: boolean): void {
    if (item instanceof FilterTreeviewItem) {
      item.updateRefChecked();
    }

    this.updateCheckedOfAll();
    this.raiseSelectedChange();
  }

  raiseSelectedChange(): void {
    this.generateSelection();
    const values = this.eventParser.getSelectedChange(this);
    const selectedObjects = this.eventParser.getSelectedObjectChange(this);
    setTimeout(() => {
      this.selectedChange.next({ values, objects: selectedObjects });
    });
  }

  private createHeaderTemplateContext(): void {
    this.headerTemplateContext = {
      config: this.config,
      item: this.allItem,
      onCheckedChange: () => this.onAllCheckedChange(),
      onCollapseExpand: () => this.onAllCollapseExpand(),
      onFilterTextChange: (text: any) => this.onFilterTextChange(text),
    };
  }

  private generateSelection(): void {
    let checkedItems: TreeviewItem[] = [];
    let uncheckedItems: TreeviewItem[] = [];
    if (!isNil(this.items)) {
      const selection = TreeviewHelper.concatSelection(
        this.items,
        checkedItems,
        uncheckedItems
      );

      checkedItems = selection['checked'];
      uncheckedItems = selection['unchecked'];
    }
    this.selection = {
      checkedItems,
      uncheckedItems,
    };
  }

  private updateFilterItems(): void {
    if (this.filterText !== '') {
      const filterItems: TreeviewItem[] = [];
      const filterText = this.filterText.toLowerCase();
      this.items.forEach((item) => {
        const newItem = this.filterItem(item, filterText);
        if (!isNil(newItem)) {
          filterItems.push(newItem);
        }
      });
      this.filterItems = filterItems;
    } else {
      this.filterItems = this.items;
    }

    this.updateCheckedOfAll();
  }

  private filterItem(item: TreeviewItem, filterText: string): any {
    const isMatch = includes(item.text.toLowerCase(), filterText);
    if (isMatch) {
      return item;
    } else {
      if (!isNil(item.children)) {
        const children: TreeviewItem[] = [];
        item.children.forEach((child) => {
          const newChild = this.filterItem(child, filterText);
          if (!isNil(newChild)) {
            children.push(newChild);
          }
        });
        if (children.length > 0) {
          const newItem = new FilterTreeviewItem(item);
          newItem.collapsed = false;
          newItem.children = children;
          return newItem;
        }
      }
    }

    return undefined;
  }

  private updateCheckedOfAll(): void {
    let itemChecked: any = null;
    for (const filterItem of this.filterItems) {
      if (itemChecked === null) {
        itemChecked = filterItem.checked;
      } else if (itemChecked !== filterItem.checked) {
        itemChecked = undefined;
        break;
      }
    }

    if (itemChecked === null) {
      itemChecked = false;
    }

    this.allItem.checked = itemChecked;
  }

  private updateCollapsedOfAll(): void {
    let hasItemExpanded = false;
    for (const filterItem of this.filterItems) {
      if (!filterItem.collapsed) {
        hasItemExpanded = true;
        break;
      }
    }

    this.allItem.collapsed = !hasItemExpanded;
  }
}
