import { Injectable } from '@angular/core';
import { isNil } from 'lodash';
import { TreeviewItem } from '../models/tree-view-item';
import { TreeViewComponent } from '../components/tree-view/tree-view.component';

@Injectable()
export abstract class TreeviewEventParser {
  abstract getSelectedChange(component: TreeViewComponent): any[];
  abstract getSelectedObjectChange(component: TreeViewComponent): any[];
}

@Injectable()
export class DefaultTreeviewEventParser extends TreeviewEventParser {
  getSelectedChange(component: TreeViewComponent): any[] {
    const checkedItems = component.selection.checkedItems;
    if (!isNil(checkedItems)) {
      return checkedItems.map((item) => item.value);
    }

    return [];
  }

  getSelectedObjectChange(component: TreeViewComponent): any[] {
    const checkedItems = component.selection.checkedItems;
    if (!isNil(checkedItems)) {
      return checkedItems.map((item) => item);
    }

    return [];
  }
}

export interface DownlineTreeviewItem {
  item: TreeviewItem;
  parent: DownlineTreeviewItem;
}

@Injectable()
export class DownlineTreeviewEventParser extends TreeviewEventParser {
  getSelectedObjectChange(component: TreeViewComponent): any[] {
    throw new Error('Method not implemented.');
  }
  getSelectedChange(component: TreeViewComponent): any {
    const items = component.items;
    if (!isNil(items)) {
      let result: DownlineTreeviewItem[] = [];
      items.forEach((item) => {
        const links = this.getLinks(item, null);
        if (!isNil(links)) {
          result = result.concat(links);
        }
      });
      return result;
    }

    return [];
  }

  private getLinks(item: TreeviewItem, parent: any): any {
    if (!isNil(item.children)) {
      const link = {
        item,
        parent,
      };
      let result: DownlineTreeviewItem[] = [];
      item.children.forEach((child) => {
        const links = this.getLinks(child, link);
        if (!isNil(links)) {
          result = result.concat(links);
        }
      });

      return result;
    }

    if (item.checked) {
      return [
        {
          item,
          parent,
        },
      ];
    }

    return null;
  }
}

@Injectable()
export class OrderDownlineTreeviewEventParser extends TreeviewEventParser {
  private currentDownlines: DownlineTreeviewItem[] = [];
  private parser = new DownlineTreeviewEventParser();

  getSelectedChange(component: TreeViewComponent): any[] {
    const newDownlines: DownlineTreeviewItem[] =
      this.parser.getSelectedChange(component);
    if (this.currentDownlines.length === 0) {
      this.currentDownlines = newDownlines;
    } else {
      const intersectDownlines: DownlineTreeviewItem[] = [];
      this.currentDownlines.forEach((downline) => {
        let foundIndex = -1;
        const length = newDownlines.length;
        for (let i = 0; i < length; i++) {
          if (downline.item.value === newDownlines[i].item.value) {
            foundIndex = i;
            break;
          }
        }

        if (foundIndex !== -1) {
          intersectDownlines.push(newDownlines[foundIndex]);
          newDownlines.splice(foundIndex, 1);
        }
      });

      this.currentDownlines = intersectDownlines.concat(newDownlines);
    }

    return this.currentDownlines;
  }

  getSelectedObjectChange(component: TreeViewComponent): any[] {
    throw new Error('Method not implemented.');
  }
}
